@@ -64,12 +64,27 @@ class BILUOSequenceEncoderDecoder(object):
     # O is very easy confused with zero, using oscar instead in the code
     oscar = "O"
 
+    legal_set = {
+        ("B", "I"),
+        ("B", "L"),
+        ("I", "I"),
+        ("I", "L"),
+        (oscar, "B"),
+        ("L", oscar),
+        (oscar, "U"),
+        ("U", oscar),
+        ("U", "B"),
+        ("L", "U"),
+        ("U", "U"),
+    }
+
     prefix_set = set("BILU")
 
     def __init__(self, *args, **kwargs):
         self.ignore_error = kwargs.get("ignore_error", True)
 
     def parse_tag(self, tag):
+        # TODO: already replaced by inline code, remove me in later version
         if tag == self.oscar:
             return self.oscar, None
 
@@ -87,38 +102,39 @@ class BILUOSequenceEncoderDecoder(object):
     def is_prefix_legal(self, previous, current):
         node = (previous, current)
 
-        legal_set = {
-            ("B", "I"),
-            ("B", "L"),
-            ("I", "I"),
-            ("I", "L"),
-            (self.oscar, "B"),
-            ("L", self.oscar),
-            (self.oscar, "U"),
-            ("U", self.oscar),
-            ("U", "B"),
-            ("L", "U"),
-            ("U", "U"),
-        }
-
-        return node in legal_set
+        return node in self.legal_set
 
     def decode_to_offset(self, sequence):
         offset_list = []
 
-        tag_prefix_cache = []
+        last_tag_prefix = None
+        tag_length = 0
         tag_name_cache = None
 
         for index, item in enumerate(sequence):
-            prefix, tag_name = self.parse_tag(item)
+            # inline function <<< self.parse_tag(item)
+            tag = item
+            if tag == self.oscar:
+                prefix = self.oscar
+                tag_name = None
+            else:
+                # set maxsplit to 1, so raw_tag_name can contains '-' char legally
+                prefix, tag_name = tag.split("-", maxsplit=1)
 
-            if not tag_prefix_cache:
+                if prefix and tag_name and prefix in self.prefix_set:
+                    pass
+                else:
+                    raise ValueError("tag: {} is not a avoid tag".format(tag))
+            # inline function >>> self.parse_tag(item)
+
+            if last_tag_prefix is None:
                 if prefix == self.oscar:
                     # ignore it
                     continue
                 elif prefix == "B":
                     tag_name_cache = tag_name
-                    tag_prefix_cache.append(prefix)
+                    last_tag_prefix = prefix
+                    tag_length += 1
                 elif prefix == "U":
                     offset_list.append((index, index + 1, tag_name))
                 else:
@@ -131,8 +147,6 @@ class BILUOSequenceEncoderDecoder(object):
                     else:
                         continue
             else:
-                last_tag_prefix = tag_prefix_cache[-1]
-
                 if not self.is_prefix_legal(last_tag_prefix, prefix):
                     raise TagSetDecodeError(
                         "sequence: {} is not a valid tag sequence".format(
@@ -143,11 +157,12 @@ class BILUOSequenceEncoderDecoder(object):
                 if prefix == "L":
                     if tag_name_cache == tag_name:
                         offset_list.append(
-                            (index - len(tag_prefix_cache), index + 1, tag_name_cache)
+                            (index - tag_length, index + 1, tag_name_cache)
                         )
 
                         # clean up
-                        tag_prefix_cache = []
+                        last_tag_prefix = None
+                        tag_length = 0
                         tag_name_cache = None
                     else:
                         raise TagSetDecodeError(
@@ -157,7 +172,8 @@ class BILUOSequenceEncoderDecoder(object):
                         )
                 elif prefix == "I":
                     if tag_name_cache == tag_name:
-                        tag_prefix_cache.append(prefix)
+                        last_tag_prefix = prefix
+                        tag_length += 1
                     else:
                         raise TagSetDecodeError(
                             "sequence: {} is not a valid tag sequence".format(
@@ -168,19 +184,20 @@ class BILUOSequenceEncoderDecoder(object):
         return offset_list
 
     def to_offset(self, sequence, text, **kwargs):
-        seq = Document(text)
+        span_set = SpanSet(
+            [
+                Span(offset[0], offset[1], offset[2])
+                for offset in self.decode_to_offset(sequence)
+            ]
+        )
 
-        plain_offset_list = self.decode_to_offset(sequence)
-
-        for offset in plain_offset_list:
-            seq.span_set.append(Span(offset[0], offset[1], offset[2]))
+        label = kwargs.pop("label", None)
+        id_ = kwargs.pop("id", None)
+        extra_attr = kwargs
 
+        seq = Document(text, span_set, id_, label, extra_attr)
         seq.span_set.bind(seq)
 
-        seq.label = kwargs.pop("label", None)
-        seq.id = kwargs.pop("id", None)
-        seq.extra_attr = kwargs
-
         return seq
 
 
