@@ -11,6 +11,8 @@ from tokenizer_tools.tagset.offset.span import Span
 class DocumentPattern(Document):
     def __init__(self, *args, **kwargs):
         super().__init__(*args, **kwargs)
+
+        # docs which can reduce to this pattern
         self.docs = []
 
     @classmethod
@@ -47,7 +49,7 @@ class DocumentPattern(Document):
     def _copy_structure_from_doc(cls, doc) -> "DocumentPattern":
         dp = cls(copy.deepcopy(doc.text))
 
-        attrs_need_be_copied = ["domain", "intent", "function", "sub_function"]
+        attrs_need_be_copied = ["label", "domain", "intent", "function", "sub_function"]
 
         for attr in attrs_need_be_copied:
             setattr(dp, attr, getattr(doc, attr))
@@ -60,7 +62,7 @@ class DocumentPattern(Document):
     def _copy_structure_as_doc(self) -> Document:
         doc = Document(copy.deepcopy(self.text))
 
-        attrs_need_be_copied = ["domain", "intent", "function", "sub_function"]
+        attrs_need_be_copied = ["label", "domain", "intent", "function", "sub_function"]
 
         for attr in attrs_need_be_copied:
             setattr(doc, attr, getattr(self, attr))
