@@ -1,6 +1,7 @@
 from collections import Counter
-import pandas as pd
-import plotly.express as px
+
+import pandas as pd  # pytype: disable=import-error
+import plotly.express as px  # pytype: disable=import-error
 
 
 class PlottableCounter(Counter):
