@@ -56,7 +56,7 @@ class SpanSet(List[Span]):
 
         return True, []
 
-    def bind(self, host: "Sequence"):
+    def bind(self, host):
         self.host = host
 
         for span in self:
